#include <zephyr/logging/log.h>
#include <bootutil/image.h>
#include "image_hash_utils.h"

#define LOG_MODULE_NAME image_hash_utils
LOG_MODULE_REGISTER(LOG_MODULE_NAME);

int8_t get_image_hash(uint8_t* data, size_t imageareasize, size_t* binsize, uint8_t hash[HASHSIZE])
{
    int8_t res = -1;

    *binsize = 0;

    if (data != NULL) {
        struct image_header* header = (struct image_header*)data;

        if (header->ih_magic == IMAGE_MAGIC) {
            size_t tlvoffset = header->ih_hdr_size + header->ih_img_size + header->ih_protect_tlv_size;

            if (tlvoffset < imageareasize) {
                uint8_t* tlvstart = data + tlvoffset;
                struct image_tlv_info* tlvinfo = (struct image_tlv_info*)tlvstart;

                if (tlvinfo->it_magic == IMAGE_TLV_INFO_MAGIC) {
                    size_t imagesize = tlvoffset + tlvinfo->it_tlv_tot;

                    if (imagesize <= imageareasize ) {
                    struct image_tlv* tlv = (struct image_tlv*)((uint8_t*)tlvinfo + sizeof(struct image_tlv_info));
                    uint8_t* end = data + imagesize;


                    bool found = false;

                    LOG_INF("TLV area found");

                    do {
                        if (tlv->it_type == IMAGE_TLV_SHA256) {
                            uint8_t* hashdata = (uint8_t*)tlv + sizeof(struct image_tlv);

                            LOG_INF("Sha256 TLV found");

                            memcpy(hash, hashdata, HASHSIZE);
                            *binsize = imagesize;
                            found = true;
                            res = 0;
                        }

                        tlv = (struct image_tlv*)((uint8_t*)tlv + sizeof(struct image_tlv) + tlv->it_len);
                    } while(!found && ((uint8_t*)tlv < end));
                } else {
                    LOG_ERR("Calculated image size is larger than image area");
                    res = -5;
                }
                } else {
                    LOG_ERR("TLV magic not found");
                    res = -4;
                }
            } else {
                LOG_ERR("Calculated tlv offset points part end of imagearea");
                res = -3;
            }
        } else {
            LOG_ERR("header magic is not correct");
            res = -2;
        }
    }

    return res;
}

